@@ -14,16 +14,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import argparse
 import logging
 
-from cliff import command
-from cliff import lister
-from cliff import show
+from osc_lib.command import command
 import six
 
 from designateclient import utils
+from designateclient.v2.cli import common
 from designateclient.v2.utils import get_all
 
+
 LOG = logging.getLogger(__name__)
 
 
@@ -34,7 +35,15 @@ def _format_recordset(recordset):
     return recordset
 
 
-class ListRecordSetsCommand(lister.Lister):
+def _has_project_id(data):
+    if len(data) < 1:
+        return False
+    if 'project_id' in data[0]:
+        return True
+    return False
+
+
+class ListRecordSetsCommand(command.Lister):
     """List recordsets"""
 
     columns = ['id', 'name', 'type', 'records', 'status', 'action']
@@ -55,12 +64,16 @@ class ListRecordSetsCommand(lister.Lister):
         parser.add_argument('--action', help="RecordSet Action",
                             required=False)
 
-        parser.add_argument('zone_id', help="Zone ID")
+        parser.add_argument('zone_id', help="Zone ID. To list all"
+                            " recordsets specify 'all'")
+
+        common.add_all_common_options(parser)
 
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.dns
+        common.set_all_common_headers(client, parsed_args)
 
         criterion = {}
         if parsed_args.type is not None:
@@ -86,8 +99,16 @@ class ListRecordSetsCommand(lister.Lister):
 
         cols = self.columns
 
-        data = get_all(client.recordsets.list, args=[parsed_args.zone_id],
-                       criterion=criterion)
+        if parsed_args.zone_id == 'all':
+            data = get_all(client.recordsets.list_all_zones,
+                           criterion=criterion)
+            cols.insert(2, 'zone_name')
+        else:
+            data = get_all(client.recordsets.list, args=[parsed_args.zone_id],
+                           criterion=criterion)
+
+        if client.session.all_projects and _has_project_id(data):
+            cols.insert(1, 'project_id')
 
         for i, rs in enumerate(data):
             data[i] = _format_recordset(rs)
@@ -95,7 +116,7 @@ class ListRecordSetsCommand(lister.Lister):
         return cols, (utils.get_item_properties(s, cols) for s in data)
 
 
-class ShowRecordSetCommand(show.ShowOne):
+class ShowRecordSetCommand(command.ShowOne):
     """Show recordset details"""
 
     def get_parser(self, prog_name):
@@ -104,40 +125,59 @@ class ShowRecordSetCommand(show.ShowOne):
         parser.add_argument('zone_id', help="Zone ID")
         parser.add_argument('id', help="RecordSet ID")
 
+        common.add_all_common_options(parser)
+
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.dns
+        common.set_all_common_headers(client, parsed_args)
         data = client.recordsets.get(parsed_args.zone_id, parsed_args.id)
 
         _format_recordset(data)
         return six.moves.zip(*sorted(six.iteritems(data)))
 
 
-class CreateRecordSetCommand(show.ShowOne):
+class CreateRecordSetCommand(command.ShowOne):
     """Create new recordset"""
 
+    log = logging.getLogger('deprecated')
+
     def get_parser(self, prog_name):
         parser = super(CreateRecordSetCommand, self).get_parser(prog_name)
 
         parser.add_argument('zone_id', help="Zone ID")
         parser.add_argument('name', help="RecordSet Name")
-        parser.add_argument('--records', help="RecordSet Records",
-                            nargs='+', required=True)
+        req_group = parser.add_mutually_exclusive_group(required=True)
+        req_group.add_argument(
+            '--records',
+            help=argparse.SUPPRESS,
+            nargs='+')
+        req_group.add_argument(
+            '--record',
+            help="RecordSet Record, repeat if necessary",
+            action='append')
         parser.add_argument('--type', help="RecordSet Type", required=True)
         parser.add_argument('--ttl', type=int, help="Time To Live (Seconds)")
         parser.add_argument('--description', help="Description")
 
+        common.add_all_common_options(parser)
+
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.dns
+        common.set_all_common_headers(client, parsed_args)
 
+        all_records = parsed_args.record or parsed_args.records
+        if parsed_args.records:
+            self.log.warning(
+                "Option --records is deprecated, use --record instead.")
         data = client.recordsets.create(
             parsed_args.zone_id,
             parsed_args.name,
             parsed_args.type,
-            parsed_args.records,
+            all_records,
             description=parsed_args.description,
             ttl=parsed_args.ttl)
 
@@ -145,7 +185,7 @@ class CreateRecordSetCommand(show.ShowOne):
         return six.moves.zip(*sorted(six.iteritems(data)))
 
 
-class SetRecordSetCommand(show.ShowOne):
+class SetRecordSetCommand(command.ShowOne):
     """Set recordset properties"""
 
     def get_parser(self, prog_name):
@@ -153,7 +193,15 @@ class SetRecordSetCommand(show.ShowOne):
 
         parser.add_argument('zone_id', help="Zone ID")
         parser.add_argument('id', help="RecordSet ID")
-        parser.add_argument('--records', help="Records", nargs='+')
+        req_group = parser.add_mutually_exclusive_group()
+        req_group.add_argument(
+            '--records',
+            help=argparse.SUPPRESS,
+            nargs='+')
+        req_group.add_argument(
+            '--record',
+            help="RecordSet Record, repeat if necessary",
+            action='append')
 
         description_group = parser.add_mutually_exclusive_group()
         description_group.add_argument('--description', help="Description")
@@ -163,6 +211,8 @@ class SetRecordSetCommand(show.ShowOne):
         ttl_group.add_argument('--ttl', type=int, help="TTL")
         ttl_group.add_argument('--no-ttl', action='store_true')
 
+        common.add_all_common_options(parser)
+
         return parser
 
     def take_action(self, parsed_args):
@@ -178,10 +228,16 @@ class SetRecordSetCommand(show.ShowOne):
         elif parsed_args.ttl:
             data['ttl'] = parsed_args.ttl
 
+        all_records = parsed_args.record or parsed_args.records
         if parsed_args.records:
-            data['records'] = parsed_args.records
+            self.log.warning(
+                "Option --records is deprecated, use --record instead.")
+
+        if all_records:
+            data['records'] = all_records
 
         client = self.app.client_manager.dns
+        common.set_all_common_headers(client, parsed_args)
 
         updated = client.recordsets.update(
             parsed_args.zone_id,
@@ -193,7 +249,7 @@ class SetRecordSetCommand(show.ShowOne):
         return six.moves.zip(*sorted(six.iteritems(updated)))
 
 
-class DeleteRecordSetCommand(command.Command):
+class DeleteRecordSetCommand(command.ShowOne):
     """Delete recordset"""
 
     def get_parser(self, prog_name):
@@ -202,10 +258,16 @@ class DeleteRecordSetCommand(command.Command):
         parser.add_argument('zone_id', help="Zone ID")
         parser.add_argument('id', help="RecordSet ID")
 
+        common.add_all_common_options(parser)
+
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.dns
-        client.recordsets.delete(parsed_args.zone_id, parsed_args.id)
+        common.set_all_common_headers(client, parsed_args)
+        data = client.recordsets.delete(parsed_args.zone_id, parsed_args.id)
 
         LOG.info('RecordSet %s was deleted', parsed_args.id)
+
+        _format_recordset(data)
+        return six.moves.zip(*sorted(six.iteritems(data)))
